@@ -1,8 +1,10 @@
+import asyncio
 import hashlib
 import os
 import stat
 import urllib.parse
 from datetime import datetime, timezone
+from pathlib import Path
 from typing import AsyncGenerator, Callable, Dict, List, Optional
 
 import aiofiles
@@ -11,18 +13,36 @@ from pydantic import BaseModel
 
 # Define excluded directories and file patterns
 IGNORE_PATTERNS = [
+    # IDE
     ".omega",
+    ".vscode",
+    ".idea",
+    ".DS_Store",
+    # Environment
     ".git",
-    ".venv/",
     ".env",
-    "__pycache__",
     ".local",
+    # Python
+    "venv",
+    ".venv",
+    "__pycache__",
     ".jupyter",
     ".ipython",
     ".ipynb_checkpoints",
+    # Node
     ".npm",
     "node_modules",
-    ".DS_Store",
+]
+
+# Define patterns that should be synced (build results for Node apps)
+ALLOW_PATTERNS = [
+    "dist",
+    "build",
+    "out",
+    "public",
+    ".next",
+    ".nuxt",
+    ".svelte-kit",
 ]
 
 # Constants for file operations
@@ -59,45 +79,103 @@ class FileSyncManager:
     """
 
     @staticmethod
-    async def create_snapshot(directory: str) -> FilesSnapshotModel:
-        """
-        Generate a snapshot of all files in a directory.
-        Returns a FileSnapshotModel with a list of files and their metadata.
-        Skips ignored directories early in the traversal process for better performance.
-        """
-        files_snapshot = []
+    async def get_ignorepatterns(directory: str) -> List[str]:
+        ignore_patterns = []
 
+        # Read .omegaignore file if it exists
+        if os.path.exists(omegaignore_path := os.path.join(directory, ".omega", ".omegaignore")):
+            with open(omegaignore_path, "r") as f:
+                ignore_patterns = split_text(f.read())
+
+        # Read .gitignore file if it exists
+        elif os.path.exists(gitignore_path := os.path.join(directory, ".gitignore")):
+            with open(gitignore_path, "r") as f:
+                ignore_patterns = split_text(f.read())
+
+        # Read .gitignore_Python or .gitignore_Node
+        else:
+            if os.path.exists(os.path.join(directory, ".omega")):
+                languages = ["Python", "Node"]
+                # writing standard .gitignore for Python or Node
+                for language in languages:
+                    language_ignore_path = os.path.join(
+                        directory, ".omega", f".gitignore_{language}"
+                    )
+                    if not os.path.exists(language_ignore_path):
+                        # Load standard .gitignore for Python or Node
+                        std_gitignore_text = await get_standard_gitignore(language)
+                        if std_gitignore_text:
+                            with open(language_ignore_path, "w") as f:
+                                f.write(std_gitignore_text)
+
+                # reading standard .gitignore for Python or Node
+                for language in languages:
+                    language_ignore_path = os.path.join(
+                        directory, ".omega", f".gitignore_{language}"
+                    )
+                    if os.path.exists(language_ignore_path):
+                        with open(language_ignore_path, "r") as f:
+                            std_ignore_patterns = split_text(f.read())
+                            ignore_patterns.extend(std_ignore_patterns)
+
+        # add default ignore patterns
+        ignore_patterns.extend(IGNORE_PATTERNS)
+
+        # exclude some important patterns (we need build results for Node)
+        ignore_patterns = [
+            p
+            for p in ignore_patterns
+            if p not in ALLOW_PATTERNS and p.strip("/") not in ALLOW_PATTERNS
+        ]
+
+        return ignore_patterns
+
+    @staticmethod
+    async def create_snapshot(directory: str) -> FilesSnapshotModel:
+        """Create a snapshot of all files in the directory, respecting .gitignore rules."""
+        ignore_patterns = await FileSyncManager.get_ignorepatterns(directory)
+
+        snapshot_tasks = []
+
+        # Walk through directory
         for root, dirs, filenames in os.walk(directory, topdown=True):
             # Filter out ignored directories early - modifying dirs in place affects traversal
-            dirs[:] = [d for d in dirs if not any(pattern in d for pattern in IGNORE_PATTERNS)]
-
-            # Process files in current directory
+            dirs[:] = [
+                d
+                for d in dirs
+                if not any(matches_gitignore_pattern(d, pattern) for pattern in ignore_patterns)
+            ]
             for filename in filenames:
-                # Skip ignored files
-                if any(pattern in filename for pattern in IGNORE_PATTERNS):
+                abs_path = os.path.join(root, filename)
+                rel_path = os.path.relpath(abs_path, directory)
+
+                # Skip if file matches any gitignore pattern
+                if any(matches_gitignore_pattern(rel_path, pattern) for pattern in ignore_patterns):
                     continue
 
-                file_path = os.path.join(root, filename)
-                rel_path = os.path.relpath(file_path, directory)
+                # Create task for processing file
+                snapshot_tasks.append(FileSyncManager.get_file_metadata(directory, rel_path))
 
-                # Skip if the full relative path matches any ignore pattern
-                if any(pattern in rel_path for pattern in IGNORE_PATTERNS):
-                    continue
-
-                # Get file stats
-                stats = os.stat(file_path)
-                size = stats.st_size
-                modified = datetime.fromtimestamp(stats.st_mtime, tz=timezone.utc).isoformat()
-
-                # Calculate file hash
-                file_hash = await FileSyncManager.calculate_file_hash(file_path)
-
-                files_snapshot.append(
-                    FileMetadataModel(path=rel_path, size=size, hash=file_hash, modified=modified)
-                )
+        # Process all files concurrently
+        files_snapshot = await asyncio.gather(*snapshot_tasks)
 
         return FilesSnapshotModel(files=files_snapshot)
 
+    @staticmethod
+    async def get_file_metadata(directory: str, rel_path: str) -> FileMetadataModel:
+        # Get absolute path
+        abs_path = os.path.join(directory, rel_path)
+
+        # Get file stats
+        stats = os.stat(abs_path)
+        size = stats.st_size
+        modified = datetime.fromtimestamp(stats.st_mtime, tz=timezone.utc).isoformat()
+
+        # Calculate file hash
+        file_hash = await FileSyncManager.calculate_file_hash(abs_path)
+
+        return FileMetadataModel(path=rel_path, size=size, hash=file_hash, modified=modified)
+
     @staticmethod
     def compare_snapshots(
         client_snapshot: FilesSnapshotModel, server_snapshot: FilesSnapshotModel
@@ -478,3 +556,42 @@ class FileSyncManager:
         - Returns a string like: %2Esomefolder%2F%2Esomefile
         """
         return urllib.parse.quote(path, safe="").replace(".", "%2E")
+
+
+def split_text(text: str) -> List[str]:
+    return [line.strip() for line in text.splitlines() if line.strip() and not line.startswith("#")]
+
+
+def matches_gitignore_pattern(path: str, pattern: str) -> bool:
+    """
+    Check if a path matches a gitignore-style pattern.
+    Handles directory patterns (ending with '/') and wildcards correctly.
+    """
+    # Convert path to forward slashes for consistent matching
+    path = path.replace(os.sep, "/")
+
+    # Handle directory patterns
+    if pattern.endswith("/"):
+        pattern = pattern.rstrip("/")
+        return path == pattern or path.startswith(pattern + "/")
+
+    # For file patterns, check if the pattern matches any part of the path
+    # This matches git's behavior where patterns can match in any subdirectory
+    path_parts = path.split("/")
+    return any(Path(part).match(pattern) for part in path_parts)
+
+
+async def get_standard_gitignore(language: str) -> str:
+    # language: Python | Node
+    std_gitignore_url = (
+        f"https://raw.githubusercontent.com/github/gitignore/refs/heads/main/{language}.gitignore"
+    )
+    try:
+        async with httpx.AsyncClient() as client:
+            response = await client.get(std_gitignore_url)
+            if response.is_success:
+                return response.text
+    except Exception:
+        pass
+
+    return ""
